@@ -87,17 +87,18 @@ class SaveFilterMixin(SingleTableMixin, SaveFilterMixinNT):
 
 class InlineFormsetMixin:
     ''' This mixin allows for multiple formset factories to be injected and processed in a form view '''
-    factories = [] # list of dictionaries that must contain the key factory and the value of a formset factory instance, helper and herder are optional
+    factories = [] # list of dictionaries that must contain the key factory and the value of a formset factory instance, helper and header are optional
     form_helper = None
     template_name = 'django_aux/inline-formset.html'
+    max_extra = 10
 
     def get_current_extra(self):
         extra = self.request.GET.get('extra')
         if not extra:
             return 1
         extra = int(extra)
-        if extra > 10:
-            return 10
+        if extra > self.max_extra:
+            return self.max_extra
         else:
             return extra 
 
@@ -318,7 +319,9 @@ class PlotlyMixin:
     plot_width = 1000
     plot_height = None
     max_records = 1_000_000_000
-
+    remove_null_agg = True # whether or not to remove null values post aggregation
+    plot_title = '' 
+    include_id_in_agg_choices = True
 
     def check_qs_count(self):
         """ Performs a check to see if the queryset count is greater than the max allowed records
@@ -391,7 +394,7 @@ class PlotlyMixin:
         """          
         agg_func = self.yd.get('agg_expr')
         akwargs = {f'{self.y}':agg_func}
-        akwargs['N'] = Count('id')
+        akwargs['N'] = Count('id', distinct=True)
         return akwargs 
 
     def get_grouped_qs(self):
@@ -409,6 +412,8 @@ class PlotlyMixin:
         fargs = [~Q(**{f'{self.x}':None})]
         if self.color != None: fargs.append(~Q(**{f'{self.color}':None}))
         if self.aggregate_by != None: fargs.append(~Q(**{f'{self.aggregate_by}':None}))
+        if self.remove_null_agg:
+            fargs.append(~Q(**{self.y:None}))
         gqs = qs.values(*vargs, **vkwargs).annotate(**akwargs).filter(*fargs, **fkwargs).order_by(self.x)
         return gqs
 
@@ -456,15 +461,25 @@ class PlotlyMixin:
         args, kwargs, plot_obj = self.get_px_args_kwargs_obj()
         fig = plot_obj(*args, **kwargs)
         fig.update_layout(
-            xaxis_title=self.x_verbose, yaxis_title=self.y_verbose
+            xaxis_title=self.x_verbose, yaxis_title=self.y_verbose,
         )
-        return offline.plot(fig, auto_open=False, output_type="div")
+        fig.update_layout(title_text=self.plot_title, title_x=0.5)
+        return fig
 
+    def get_fig_offline(self):
+        fig = self.get_fig()
+        if not fig:
+            return None
+        else:
+            return offline.plot(fig, auto_open=False, output_type="div")
 
     def get_filterset_kwargs(self, filterset_class):
         kwargs = super().get_filterset_kwargs(filterset_class)
         x_choices = self.X_CHOICES
         agg_choices = [(None,'------')]
+        if self.include_id_in_agg_choices:
+            agg_choices.append(('id','ID'))
+            self.CHOICE_VALUES_MAP.update({'id':'id'})
         color_choices = [(None,'------')]
         if not hasattr(self, 'COLOR_CHOICES'):
             color_choices += self.X_CHOICES
@@ -496,7 +511,7 @@ class PlotlyMixin:
 
     def get_context_data(self, **kwargs):
         context = super().get_context_data(**kwargs)
-        context['fig'] = self.get_fig()
+        context['fig'] = self.get_fig_offline()
         context['plot_df'] = self.plot_df
         return context
 
