@@ -12,6 +12,7 @@ from django.test.client import RequestFactory
 from django.test import Client
 from django.urls import path, reverse
 from django.contrib.sessions.middleware import SessionMiddleware
+from .utils import *
 
 #------------Django-Aux TESTS------------
 
@@ -127,4 +128,78 @@ class TestRedirectPrevMixin(TestCase):
         self.assertEqual(response.wsgi_request.session.get('next'), 'final-destination')
 
         response = self.client.get('/person-create', HTTP_REFERER='person-delete')
-        self.assertEqual(response.wsgi_request.session.get('next'), '/person-lookup')
\ No newline at end of file
+        self.assertEqual(response.wsgi_request.session.get('next'), '/person-lookup')
+
+
+
+class TestInlineFormsetMixin(TestCase):
+    
+    def test_get_current_extra(self):
+        ifm = InlineFormsetMixin()
+        ifm.max_extra = 10
+        ifm.request = FakeRequest(GET = {'extra':None})
+        self.assertEqual(ifm.get_current_extra(), 1)
+        ifm.request = FakeRequest(GET = {'extra':1})
+        self.assertEqual(ifm.get_current_extra(), 1)
+        ifm.request = FakeRequest(GET = {'extra':2})
+        self.assertEqual(ifm.get_current_extra(), 2)
+        # try one that exceeds max
+        ifm.request = FakeRequest(GET = {'extra':11})
+        self.assertEqual(ifm.get_current_extra(), 10) 
+
+    def test_add_factories_to_context(self):
+        ifm = InlineFormsetMixin()
+        ifm.factories = [
+            {'factory':FakeFormsetFactory(), 'helper':None, 'header':None},
+            {'factory':FakeFormsetFactory(), 'helper':None, 'header':None},
+        ]
+        ifm.request = FakeRequest(method='GET')
+        ifm.object = None
+        context = {}
+        ifm.add_factories_to_context(context)
+        num_factories = len(context.get('factories'))
+        self.assertEqual(num_factories, 2)
+
+
+    def test_set_extra_on_factories(self):
+        ifm = InlineFormsetMixin()
+        ifm.object = None
+        ifm.request = FakeRequest(GET = {'extra':2}, method='GET')
+        ifm.factories = [
+            {'factory':FakeFormsetFactory(), 'helper':None, 'header':None},
+            {'factory':FakeFormsetFactory(), 'helper':None, 'header':None},
+        ]
+        context = {}
+        ifm.add_factories_to_context(context)
+        ifm.set_extra_on_factories(context)
+        for fd in context.get('factories'):
+            self.assertEqual(fd.get('factory').extra, 2)
+
+    def test_add_addlines_url_to_context(self):
+        ifm = InlineFormsetMixin()
+        tests = (
+            {'extra':None, 'target':2},
+            {'extra':1, 'target':2},
+            {'extra':5, 'target':6},
+            {'extra':10, 'target':11},
+        )
+        for td in tests:
+            ifm.request = FakeRequest(GET = {'extra':td.get('extra')})
+            context = {}
+            ifm.add_addlines_url_to_context(context)
+            self.assertEqual(context['addlines_url'], f"?extra={td.get('target')}")
+
+
+    def test_add_removelines_url_to_context(self):
+        ifm = InlineFormsetMixin()
+        tests = (
+            {'extra':None, 'target':1},
+            {'extra':1, 'target':1},
+            {'extra':5, 'target':4},
+            {'extra':10, 'target':9},
+        )
+        for td in tests:
+            ifm.request = FakeRequest(GET = {'extra':td.get('extra')})
+            context = {}
+            ifm.add_removelines_url_to_context(context)
+            self.assertEqual(context['removelines_url'], f"?extra={td.get('target')}")
\ No newline at end of file
